import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:verttice_ui/src/widgets/dialogs/full_page_dialog.dart';
import 'package:verttice_ui/src/widgets/form_factory/form_factory.dart';
import 'package:verttice_ui/src/widgets/themed_app_bar.dart';
import 'package:verttice_ui/src/widgets/themed_progress.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';
import 'package:verttice_ui/src/widgets/form_factory/form_fields.dart';

class AddPageForm extends StatefulWidget {
  final List<FormItem> columns;
  final Map<String, dynamic>? formState;
  final Future<void> Function({
    required BuildContext context,
    required Map<String, dynamic> formState,
  }) submitForm;
  final String formDisplayName;
  final Future<Map<String, dynamic>> Function({
    required BuildContext context,
    required Map<String, dynamic> currentState,
    required String name,
    dynamic data,
  })? stateChangesProcessor;

  const AddPageForm({
    Key? key,
    required this.columns,
    required this.submitForm,
    required this.formDisplayName,
    this.formState,
    this.stateChangesProcessor,
  }) : super(key: key);

  @override
  _AddPageFormState createState() => _AddPageFormState();
}

class _AddPageFormState extends State<AddPageForm> {
  late StateSetter _setState;
  Map<String, dynamic> _newFormState = {};
  bool _isSaveingForm = false;

  @override
  void initState() {
    super.initState();
    _newFormState = widget.formState ?? {};
  }

  _onChanged(String name, dynamic data) async {
    if (widget.stateChangesProcessor != null) {
      final updatedState = await widget.stateChangesProcessor!(
        context: context,
        currentState: _newFormState,
        name: name,
        data: data,
      );
      _setState(() {
        _newFormState = updatedState;
      });
      return;
    }
    _setState(() {
      _newFormState[name] = data;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Tooltip(
      message: "Add ${widget.formDisplayName}",
      child: IconButton(
        onPressed: () async {
          await fullPageDialog(
            context: context,
            builder: (BuildContext context) =>
                StatefulBuilder(builder: (BuildContext context, setState) {
              _setState = setState;

              return Scaffold(
                appBar: ThemedAppBar(
                  actions: [
                    Tooltip(
                      message: "Clear Form",
                      child: IconButton(
                        onPressed: () async {
                          _setState(() {
                            _newFormState = {};
                          });
                        },
                        icon: ThemedIcons.clear(),
                      ),
                    ),
                    if (_isSaveingForm)
                      Row(
                        children: const [
                          SizedBox(
                            width: 10,
                          ),
                          ThemedProgress(),
                        ],
                      ),
                    Tooltip(
                      message: "Save New ${widget.formDisplayName}",
                      child: IconButton(
                        onPressed: () async {
                          _setState(() {
                            _isSaveingForm = true;
                          });
                          try {
                            Map<String, dynamic> defData = {};
                            for (var element in widget.columns) {
                              if (element.field.defaultValue != null) {
                                defData[element.field.name] =
                                    element.field.defaultValue;
                              }
                            }

                            await widget.submitForm(
                              context: context,
                              formState: {...defData, ..._newFormState},
                            );
                          } catch (e) {
                            ThemedToast.error(
                              context: context,
                              message: "FAILED TO SAVE FORM STATE",
                            );
                          } finally {
                            _setState(() {
                              _isSaveingForm = false;
                            });
                            Navigator.pop(context);
                          }
                        },
                        icon: ThemedIcons.check(),
                      ),
                    ),
                  ],
                  title: "Add ${widget.formDisplayName}",
                ),
                body: SingleChildScrollView(
                  child: FormFactory(
                    formState: _newFormState,
                    formItems: widget.columns,
                    onChanged: _onChanged,
                    gridView: true,
                  ),
                ),
              );
            }),
          );
        },
        icon: ThemedIcons.add(),
      ),
    );
  }
}
