import 'package:verttice_logic/verttice_logic.dart';
import "package:verttice_ui/src/const/box_sizes.dart";
import "package:verttice_ui/src/const/icons.dart";
import "package:flutter/material.dart";
import 'package:flutter/services.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';

class DateFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final int? minDate;
  final int? maxDate;
  final Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;

  const DateFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.value,
    required this.onChanged,
    this.minDate,
    this.maxDate,
    this.state,
  }) : super(key: key);

  @override
  _DateFormState createState() => _DateFormState();
}

class _DateFormState extends State<DateFormFactory> {
  int _value = 0;
  int? _maxDate;
  int? _minDate;

  @override
  void initState() {
    super.initState();
    _maxDate = widget.maxDate;
    _minDate = widget.minDate;
    _value = DataClassUtils.dynamicToInt(widget.value, defultValue: 0)!;
  }

  @override
  void didUpdateWidget(covariant DateFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value ||
        _minDate != widget.minDate ||
        _maxDate != widget.maxDate) {
      _maxDate = widget.maxDate;
      _minDate = widget.minDate;
      _value = DataClassUtils.dynamicToInt(widget.value, defultValue: 0)!;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
              text: _value != 0
                  ? Dates.formatDataAndTime(miliseconds: _value, time: false)
                  : ""),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      actions: [
        PhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      child: TextButton.icon(
        icon: ThemedIcons.calendar(),
        label: Text(_value != 0
            ? Dates.formatDataAndTime(miliseconds: _value, time: false)
            : ""),
        onPressed: () async {
          DateTime? date = await showDatePicker(
            context: context,
            initialDate: DateTime.fromMillisecondsSinceEpoch(
                _value != 0 ? _value : DateTime.now().millisecondsSinceEpoch),
            firstDate: _minDate != null
                ? DateTime.fromMillisecondsSinceEpoch(_minDate!)
                : DateTime(2000),
            lastDate: _maxDate != null
                ? DateTime.fromMillisecondsSinceEpoch(_maxDate!)
                : DateTime(2100),
          );

          if (date == null) {
            return;
          }

          widget.onChanged(widget.name, date.millisecondsSinceEpoch);
          widget.state?.didChange(date.millisecondsSinceEpoch);
        },
      ),
    );
  }
}
