import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import "package:verttice_ui/src/const/icons.dart";
import 'package:day_night_time_picker/day_night_time_picker.dart';
import 'package:day_night_time_picker/lib/constants.dart';
import "package:flutter/material.dart";
import 'package:flutter/services.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';
import 'package:verttice_ui/src/widgets/themed_toast.dart';

class TimeRangeFormFactory extends StatefulWidget {
  final Function(String name, dynamic value) onChanged;
  final String displayName;
  final String name;
  final dynamic value;
  final FormFieldState<dynamic>? state;

  const TimeRangeFormFactory({
    Key? key,
    required this.onChanged,
    required this.displayName,
    required this.name,
    required this.value,
    this.state,
  }) : super(key: key);

  @override
  _TimeRangeFormFactoryState createState() => _TimeRangeFormFactoryState();
}

class _TimeRangeFormFactoryState extends State<TimeRangeFormFactory> {
  var _value = <String, int?>{};

  @override
  void initState() {
    super.initState();
    _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
        .cast<String, int?>();
  }

  @override
  void didUpdateWidget(covariant TimeRangeFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    final value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value);
    if (!const MapEquality().equals(_value, value)) {
      _value = DataClassUtils.dynamicToMapStringIntOrNull(widget.value)
          .cast<String, int?>();
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      onCopied: () async {
        await Clipboard.setData(
          ClipboardData(
            text:
                "${_value["startTime"] != null && _value["startTime"] is int ? Dates.formatDataAndTime(miliseconds: _value["startTime"]!) : ""} - ${_value["endTime"] != null && _value["endTime"] is int ? Dates.formatDataAndTime(miliseconds: _value["endTime"]!) : ""}",
          ),
        );
        ThemedToast.info(
          context: context,
          message: "Copied to clipboard",
        );
      },
      actions: [
        PhysicalModel(
          color: Colors.grey,
          elevation: 5,
          shape: BoxShape.circle,
          child: CircleAvatar(
            backgroundColor: Colors.red,
            child: InkWell(
              child: ThemedIcons.clear(
                iconStyle: IconStyle(
                  color: Colors.white,
                ),
              ),
              onTap: () {
                widget.onChanged(widget.name, null);
                widget.state?.didChange(null);
              },
            ),
          ),
        ),
        sizedBoxW2,
      ],
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          TextButton.icon(
            onPressed: () async {
              DateTime? initialDate;
              if (_value['startTime'] != null && _value['startTime'] is int) {
                initialDate =
                    DateTime.fromMillisecondsSinceEpoch(_value['startTime']!);
              }

              showDialog(
                context: context,
                builder: (context) => AlertDialog(
                  content: createInlinePicker(
                    value: TimeOfDay.fromDateTime(
                      initialDate ?? DateTime.now(),
                    ),
                    minuteInterval: MinuteInterval.FIVE,
                    onChange: (time) {
                      final currentDate =
                          DateTime.fromMillisecondsSinceEpoch(0);
                      int timeInDate = DateTime(
                        currentDate.year,
                        currentDate.month,
                        currentDate.day,
                        time.hour,
                        time.minute,
                      ).millisecondsSinceEpoch;

                      widget.onChanged(widget.name, timeInDate);
                      widget.state?.didChange(timeInDate);
                    },
                  ),
                ),
              );
              // if (date == null) {
              //   return;
              // }

              // final choosenDate = {
              //   "startTime": date.millisecondsSinceEpoch,
              //   "endTime": DateTime(
              //     date.year,
              //     date.month,
              //     date.day,
              //     date.hour + 3,
              //     date.minute,
              //   ).millisecondsSinceEpoch,
              // };
              // widget.onChanged(widget.name, choosenDate);
              // widget.state?.didChange(choosenDate);
            },
            icon: ThemedIcons.calendar(),
            label: Text(
              _value["startTime"] != null && _value["startTime"] is int
                  ? Dates.formatDataAndTime(miliseconds: _value["startTime"]!)
                  : "Start Date",
            ),
          ),
          if (_value['endTime'] != null && _value['endTime'] is int)
            TextButton.icon(
              onPressed: () async {
                DateTime? initialDate;
                if (_value['endTime'] != null && _value['endTime'] is int) {
                  initialDate =
                      DateTime.fromMillisecondsSinceEpoch(_value['endTime']!);
                }

                showDialog(
                  context: context,
                  builder: (context) => AlertDialog(
                    content: createInlinePicker(
                      value: TimeOfDay.fromDateTime(
                        initialDate ?? DateTime.now(),
                      ),
                      minuteInterval: MinuteInterval.FIVE,
                      onChange: (time) {
                        final currentDate =
                            DateTime.fromMillisecondsSinceEpoch(0);
                        int timeInDate = DateTime(
                          currentDate.year,
                          currentDate.month,
                          currentDate.day,
                          time.hour,
                          time.minute,
                        ).millisecondsSinceEpoch;

                        widget.onChanged(widget.name, timeInDate);
                        widget.state?.didChange(timeInDate);
                      },
                    ),
                  ),
                );

                // final choosenDate = {
                //   ..._value,
                //   "endTime": date.millisecondsSinceEpoch,
                // };
                // widget.onChanged(widget.name, choosenDate);
                // widget.state?.didChange(choosenDate);
              },
              icon: ThemedIcons.calendar(),
              label: Text(
                _value["endTime"] != null && _value["endTime"] is int
                    ? Dates.formatDataAndTime(miliseconds: _value["endTime"]!)
                    : "End Date",
              ),
            ),
        ],
      ),
    );
  }
}
