import "dart:math";
import "package:collection/collection.dart";
import "package:verttice_ui/src/widgets/tables/table_frame.dart";
import "package:verttice_ui/src/widgets/themed_divider.dart";
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";
import 'package:verttice_ui/src/widgets/form_factory/form_fields.dart';

class TableData extends StatefulWidget {
  final List<Map<String, dynamic>> data;
  final List<FormItem> columns;
  final String title;
  final void Function(Map<String, dynamic>? rowData)? rowPressCallback;
  final String uniqueIdField;
  final Widget? topBarWidget;
  final double? topBarPresntageHeight;
  final List<int> rowsPerPageOptions;
  final bool showCopyRowButton;
  final List<Widget>? tableActions;
  final double? tableActionPrecentageWidth;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;

  const TableData({
    Key? key,
    required this.data,
    required this.columns,
    required this.uniqueIdField,
    this.title = "Table",
    this.rowPressCallback,
    this.rowsPerPageOptions = const [20, 50, 100],
    this.showCopyRowButton = true,
    this.topBarWidget,
    this.topBarPresntageHeight,
    this.tableActions,
    this.tableActionPrecentageWidth,
    this.selectedListCallback,
  }) : super(key: key);

  @override
  _TableDataState createState() => _TableDataState();
}

class _TableDataState extends State<TableData> {
  final double _seperator = 2.0;
  late List<Map<String, dynamic>> _tableData;
  late double _topHeightPresntage;

  @override
  void initState() {
    super.initState();
    _tableData = widget.data;
    _topHeightPresntage = (widget.topBarWidget != null)
        ? min(widget.topBarPresntageHeight ?? 0.4, 0.4)
        : 0.0;
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance?.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  @override
  void didUpdateWidget(covariant TableData oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_tableData, widget.data)) {
      _tableData = widget.data;
    }
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, constraints) {
        final double topPanelHeight =
            (constraints.maxHeight * _topHeightPresntage);
        return Stack(
          children: [
            SizedBox(
              height: constraints.maxHeight,
              width: constraints.maxWidth,
            ),
            AnimatedPositioned(
                duration: const Duration(milliseconds: 200),
                top: 0,
                child: (widget.topBarWidget != null)
                    ? SizedBox(
                        height: topPanelHeight,
                        width: constraints.maxWidth,
                        child: widget.topBarWidget,
                      )
                    : const SizedBox(
                        height: 0,
                        width: 0,
                      )),
            AnimatedPositioned(
              duration: const Duration(milliseconds: 200),
              top: topPanelHeight,
              height: _seperator,
              width: constraints.maxWidth,
              child: MouseRegion(
                cursor: SystemMouseCursors.resizeRow,
                child: GestureDetector(
                  onPanUpdate: (DragUpdateDetails dragDetail) {
                    setState(() {
                      _topHeightPresntage =
                          dragDetail.globalPosition.dy / constraints.maxHeight;
                    });
                  },
                  child: ThemedDivider(
                    height: _seperator,
                    width: constraints.maxWidth,
                  ),
                ),
              ),
            ),
            AnimatedPositioned(
              duration: const Duration(milliseconds: 300),
              top: topPanelHeight + _seperator,
              height: constraints.maxHeight - (topPanelHeight + _seperator),
              width: constraints.maxWidth,
              child: TableFrame(
                data: _tableData,
                uniqueIdField: widget.uniqueIdField,
                tableActionPrecentageWidth: widget.tableActionPrecentageWidth,
                tableActions: widget.tableActions,
                frameWidth: constraints.maxWidth,
                frameHeight:
                    constraints.maxHeight - (topPanelHeight + _seperator),
                showCopyRowButton: widget.showCopyRowButton,
                selectedListCallback: widget.selectedListCallback,
                columns: widget.columns,
                rowsPerPageOptions: widget.rowsPerPageOptions,
                title: widget.title,
                handleSelectedRow:
                    (Map<String, dynamic>? rowData, int rowIndex) {
                  if (widget.rowPressCallback != null) {
                    widget.rowPressCallback!(rowData);
                  }
                },
              ),
            )
          ],
        );
      },
    );
  }
}
