import 'package:verttice_ui/src/state/verttice_ui_state.dart';
import "package:flutter/material.dart";
import 'package:provider/provider.dart';

class ThemedText extends StatelessWidget {
  final String text;
  final TextStyle? style;
  final Alignment? alignment;
  final bool selectable;

  const ThemedText(
    this.text, {
    Key? key,
    this.style,
    this.alignment,
    this.selectable = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<VertticeUIState>(builder: (context, value, _child) {
      return LayoutBuilder(builder: (context, constraints) {
        final newStyle = TextStyle(
          background: style?.background,
          backgroundColor: style?.backgroundColor,
          decoration: style?.decoration,
          decorationColor: style?.decorationColor,
          decorationStyle: style?.decorationStyle,
          decorationThickness: style?.decorationThickness,
          fontFamily: style?.fontFamily,
          fontFamilyFallback: style?.fontFamilyFallback,
          fontFeatures: style?.fontFeatures,
          fontSize: style?.fontSize ?? 20,
          fontStyle: style?.fontStyle,
          fontWeight: style?.fontWeight ?? FontWeight.normal,
          letterSpacing: style?.letterSpacing,
          color: value.themeColor,
          shadows: style?.shadows,
          overflow: TextOverflow.fade,
          textBaseline: style?.textBaseline,
          wordSpacing: style?.wordSpacing,
        );

        if (alignment != null) {
          return Align(
            alignment: alignment!,
            child: selectable
                ? SelectableText(
                    text,
                    style: newStyle,
                  )
                : Text(
                    text,
                    style: newStyle,
                  ),
          );
        }
        return selectable
            ? SelectableText(
                text,
                style: newStyle,
              )
            : Text(
                text,
                style: newStyle,
              );
      });
    });
  }
}
