import "package:flutter/material.dart";
import "package:fluttertoast/fluttertoast.dart";

class ThemedToast {
  ThemedToast({
    required BuildContext context,
    required String message,
    Color? color,
    Color? textColor,
    Duration? duration,
  }) {
    FToast fToast = FToast();
    fToast.init(context);

    Widget toast = Container(
      padding: const EdgeInsets.symmetric(horizontal: 24.0, vertical: 12.0),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(25.0),
        color: color ?? Colors.amber,
      ),
      child: SelectableText(
        message,
        style: TextStyle(color: textColor ?? Colors.black),
      ),
    );

    fToast.showToast(
      child: toast,
      gravity: ToastGravity.BOTTOM_LEFT,
      toastDuration: duration ?? const Duration(seconds: 2),
    );
  }

  ThemedToast.error({
    required BuildContext context,
    required String message,
    Duration? duration,
  }) {
    FToast fToast = FToast();
    fToast.init(context);

    Widget toast = Container(
      padding: const EdgeInsets.symmetric(horizontal: 24.0, vertical: 12.0),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(25.0),
        color: Colors.red,
      ),
      child: SelectableText(
        message,
        style: const TextStyle(color: Colors.white),
      ),
    );

    fToast.showToast(
      child: toast,
      gravity: ToastGravity.BOTTOM_LEFT,
      toastDuration: duration ?? const Duration(seconds: 2),
    );
  }

  ThemedToast.success({
    required BuildContext context,
    required String message,
    Duration? duration,
  }) {
    FToast fToast = FToast();
    fToast.init(context);

    Widget toast = Container(
      padding: const EdgeInsets.symmetric(horizontal: 24.0, vertical: 12.0),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(25.0),
        color: Colors.green,
      ),
      child: SelectableText(
        message,
        style: const TextStyle(
          color: Colors.white,
          fontWeight: FontWeight.bold,
        ),
      ),
    );

    fToast.showToast(
      child: toast,
      gravity: ToastGravity.BOTTOM_LEFT,
      toastDuration: duration ?? const Duration(seconds: 2),
    );
  }

  ThemedToast.warning({
    required BuildContext context,
    required String message,
    Duration? duration,
  }) {
    FToast fToast = FToast();
    fToast.init(context);

    Widget toast = Container(
      padding: const EdgeInsets.symmetric(horizontal: 24.0, vertical: 12.0),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(25.0),
        color: Colors.yellowAccent,
      ),
      child: SelectableText(
        message,
        style: const TextStyle(
          color: Colors.black,
          fontWeight: FontWeight.bold,
        ),
      ),
    );

    fToast.showToast(
      child: toast,
      gravity: ToastGravity.BOTTOM_LEFT,
      toastDuration: duration ?? const Duration(seconds: 2),
    );
  }

  ThemedToast.info({
    required BuildContext context,
    required String message,
    Duration? duration,
  }) {
    FToast fToast = FToast();
    fToast.init(context);

    Widget toast = Container(
      padding: const EdgeInsets.symmetric(horizontal: 24.0, vertical: 12.0),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(25.0),
        color: Colors.lightBlue[600],
      ),
      child: SelectableText(
        message,
        style: const TextStyle(
          color: Colors.white,
          fontWeight: FontWeight.bold,
        ),
      ),
    );

    fToast.showToast(
      child: toast,
      gravity: ToastGravity.BOTTOM_LEFT,
      toastDuration: duration ?? const Duration(seconds: 2),
    );
  }
}
